import 'package:apus_drones_mobile/constants/OrderStatus.dart';
import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:apus_drones_mobile/constants/Constants.dart';
import 'package:apus_drones_mobile/models/OrderItemModel.dart';
import 'package:apus_drones_mobile/models/OrderModel.dart';
import 'package:apus_drones_mobile/screens/Appbar.dart';
import 'package:apus_drones_mobile/widgets/BotNavBar.dart';
import 'package:apus_drones_mobile/widgets/Fee.dart';
import 'package:apus_drones_mobile/widgets/MapView.dart';
import 'package:apus_drones_mobile/widgets/Price.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:intl/intl.dart';

class OrderDetailScreen extends StatelessWidget {
  static final String routeName = 'Order Detail';
  final OrderModel model;
  const OrderDetailScreen({Key? key, required this.model}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      bottomNavigationBar: BotNavBar(role: UserType.CUSTOMER),
      appBar: Appbar(
        title: "Pedido #${this.model.id}",
      ),
      body: Padding(
        padding: const EdgeInsets.only(top: 35.0),
        child: SingleChildScrollView(
          child: Center(
            child: Padding(
              padding:
                  const EdgeInsets.symmetric(vertical: 20.0, horizontal: 10.0),
              child: Column(
                children: [
                  Chip(
                    label: Text(this.model.status.label),
                    backgroundColor: Colors.green.shade100,
                    shape: RoundedRectangleBorder(
                        side: BorderSide(color: Colors.green),
                        borderRadius: BorderRadius.circular(16)),
                  ),
                  Container(
                    padding: EdgeInsets.symmetric(horizontal: 30),
                    margin: EdgeInsets.symmetric(vertical: 35),
                    child: ListTile(
                      title: Center(
                          child: Text(this.model.partner.name,
                              style: TextStyle(fontSize: 26))),
                      leading: Container(
                        constraints: BoxConstraints.expand(
                          height: 200,
                          width: 120,
                        ),
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(4),
                            image: new DecorationImage(
                              fit: BoxFit.cover,
                              alignment: FractionalOffset.center,
                              image: new NetworkImage(
                                  this.model.partner.avatarUrl),
                            )),
                      ),
                      subtitle: Center(
                          child: Text(
                              DateFormat('dd/MM/yyyy')
                                  .format(this.model.createdAt),
                              style: TextStyle(fontSize: 20))),
                    ),
                  ),
                  FutureBuilder(
                    future: this.getOrderMap(),
                    builder: (ctx, snapshot) {
                      if (snapshot.connectionState != ConnectionState.done) {
                        return Center(
                          child: CircularProgressIndicator(),
                        );
                      } else {
                        return snapshot.data as Widget;
                      }
                    },
                  ),
                  Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 8.0),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Container(
                          child:
                              Text("Produtos:", style: TextStyle(fontSize: 24)),
                          margin:
                              EdgeInsets.only(bottom: 15, left: 15, right: 15),
                        ),
                        ...this
                            .model
                            .items
                            .map((item) => this._renderProduct(item))
                            .toList(),
                        Fee(
                            productsSumPrice: this._sumProductsPrices(),
                            deliveryFee: this.model.deliveryPrice,
                            totalPrice: this.model.orderPrice)
                      ],
                    ),
                  )
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }

  double _sumProductsPrices() {
    return this.model.items.fold(0, (acc, item) => item.price + acc);
  }

  Future<Widget> getOrderMap() async {
    final statusesWithMap = [
      OrderStatus.IN_FLIGHT,
      OrderStatus.DELIVERED,
    ];

    Widget content = statusesWithMap.contains(this.model.status)
        ? await this.mapWidget()
        : this.beforeMapTextWidget();

    return Container(
      child: content,
      margin: EdgeInsets.fromLTRB(0, 0, 0, 32),
    );
  }

  List<Polyline> getDronePath() {
    if (this.model.deliveryAddress == null || this.model.shopAddress == null) {
      return [];
    }

    final deliveryAddressCoords = LatLng(
        this.model.deliveryAddress!.lat, this.model.deliveryAddress!.lng);
    final shopAddressCoords =
        LatLng(this.model.shopAddress!.lat, this.model.shopAddress!.lng);

    final polyline = Polyline(
        polylineId: PolylineId("first"),
        points: [deliveryAddressCoords, shopAddressCoords],
        width: 2,
        color: Colors.red);
    return [polyline];
  }

  Future<Widget> mapWidget() async {
    final polylines = getDronePath();
    return Container(
      decoration: BoxDecoration(borderRadius: BorderRadius.circular(10)),
      child: MapView(
        polylines: polylines,
        initialPosition: polylines.isEmpty ? null : polylines[0].points[0],
        markers: await getMapMarkers(polylines),
      ),
    );
  }

  Future<List<Marker>> getMapMarkers(List<Polyline> polylines) async {
    if (polylines.isEmpty) {
      return [];
    }

    Future<BitmapDescriptor> shopIcon = BitmapDescriptor.fromAssetImage(
        ImageConfiguration(size: Size(1, 1)),
        'lib/assets/img/delivery_drone_icon.png');
    Future<BitmapDescriptor> customerIcon = BitmapDescriptor.fromAssetImage(
        ImageConfiguration(size: Size(1, 1)),
        'lib/assets/img/customer_icon.png');

    List<BitmapDescriptor> icons = await Future.wait([shopIcon, customerIcon]);

    final shopMarker = Marker(
        markerId:
            MarkerId('${model.shopAddress!.lat}|${model.shopAddress!.lng}'),
        position: LatLng(model.shopAddress!.lat, model.shopAddress!.lng),
        icon: icons[0]);

    final deliveryMarker = Marker(
        markerId: MarkerId(
            '${model.deliveryAddress!.lat}|${model.deliveryAddress!.lng}'),
        position:
            LatLng(model.deliveryAddress!.lat, model.deliveryAddress!.lng),
        icon: icons[1]);

    return [shopMarker, deliveryMarker];
  }

  Widget beforeMapTextWidget() {
    return Container(
        decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(10), color: Color(0xffF1F1F1)),
        child: Padding(
          padding: EdgeInsets.all(16),
          child: Center(
            child: Text(
              "assim que seu pedido for coletado por um de nossos drones você poderá acompanhar a entrega em tempo real aqui",
              style: TextStyle(
                fontStyle: FontStyle.italic,
                color: Color.fromRGBO(61, 61, 61, 0.75),
                fontSize: 18,
                letterSpacing: 1.25,
              ),
              textAlign: TextAlign.center,
            ),
          ),
        ));
  }

  ListTile _renderProduct(OrderItemModel item) {
    return ListTile(
      title: Text(item.product.name, style: TextStyle(fontSize: 20)),
      leading: Container(
        height: 60,
        width: 60,
        decoration: BoxDecoration(
          image: DecorationImage(
            fit: BoxFit.cover,
            alignment: FractionalOffset.center,
            image: new NetworkImage(item.product.imageUrl ?? IMG_NO_PHOTO),
          ),
        ),
      ),
      subtitle: Padding(
        padding: const EdgeInsets.only(top: 8.0),
        child: Price(price: item.price, style: TextStyle(fontSize: 18)),
      ),
    );
  }
}
